<%inherit file="base.html"/>
<%block name="title">Bottle Management</%block>


<script type="text/javascript">
me2_api_url = "${api_path}";
</script>
<h2>Base Bottles</h2>

<table id="base_bottles_table" class="table table-striped table-condensed table-bordered">
<thead>
<tr>
	<th>Identifier</th>
	<th>Actions</th>
</tr>
</thead>
<tbody>

</tbody>

</table>

<h2>Instances</h2>
<table id="bottles_table" class="table table-striped table-condensed table-bordered">
<thead>
<tr>
	<th>Identifier</th>
	<th>Base Bottle</th>
	<th>State</th>
	<th>Actions</th>
</tr>
</thead>
<tbody>

</tbody>

</table>
