<!DOCTYPE html>
<html lang="en">
<head>
<meta name="viewport" content="width=device-width, initial-scale=1.0">
<!-- Le styles -->
<link href="/static/css/bootstrap.css" rel="stylesheet">
<style>
body {
	padding-top: 60px;
	/* 60px to make the container go all the way to the bottom of the topbar */
}
</style>
<!-- Le HTML5 shim, for IE6-8 support of HTML5 elements -->
<!--[if lt IE 9]>
      <script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
    <![endif]-->
<!-- Le fav and touch icons -->
<link rel="shortcut icon" href="../assets/ico/favicon.ico">
<link rel="apple-touch-icon-precomposed" sizes="144x144"
	href="../assets/ico/apple-touch-icon-144-precomposed.png">
<link rel="apple-touch-icon-precomposed" sizes="114x114"
	href="../assets/ico/apple-touch-icon-114-precomposed.png">
<link rel="apple-touch-icon-precomposed" sizes="72x72"
	href="../assets/ico/apple-touch-icon-72-precomposed.png">
<link rel="apple-touch-icon-precomposed"
	href="../assets/ico/apple-touch-icon-57-precomposed.png">
</head>
<body>

<%def name="global_nav(entries, active)">
<div class="navbar navbar-fixed-top"> 
	<div class="navbar-inner">
		<div class="container">
			<a class="btn btn-navbar" data-toggle="collapse" data-target=".nav-collapse">
				<span class="icon-bar"></span>
				<span class="icon-bar"></span> 
				<span class="icon-bar"></span>
			</a>
			<a class="brand" href="/">mOSAIC Supervisor</a>
			<div class="nav-collapse">
				<ul class="nav">
					% for entry_id, entry_name, entry_target in entries:
						% if entry_id == active:
						<li class="active"><a href="">${entry_name}</a></li>
						% else:
						<li><a href="${entry_target}">${entry_name}</a></li>
						% endif
						
					% endfor
				</ul>
			</div>
		</div>
	</div>
</div>
</%def>

## ${global_nav([('bottle_management', 'Bottles', '/bottles'), ('instance_management', 'Instances', '/instances')], 'Bottles')}
${global_nav([],'')}

	<div class="container">
		<%block name="header">
        <h1><%block name="title"/></h1>
    	</%block>
		${self.body()}
	</div>
	<!-- /container -->

	<!-- Le javascript
    ================================================== -->
	<!-- Placed at the end of the document so the pages load faster -->
	<script src="http://ajax.googleapis.com/ajax/libs/jquery/1.7.2/jquery.min.js"></script>
	<script src="https://raw.github.com/hagino3000/jquery-jsonrpc2.0/master/jquery.jsonrpc.js"></script>
	<script src="/static/js/bootstrap.js"></script>
	<script src="/static/js/me2.js"></script>

</body>
</html>